/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.cepportal.backend;

import cz.muni.fi.cepportal.database.dto.EPLStatementDTO;
import cz.muni.fi.cepportal.database.exceptions.NonexistentEntityException;
import cz.muni.fi.cepportal.database.exceptions.PreexistingEntityException;
import cz.muni.fi.cepportal.database.services.EPLStatementService;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.inject.Named;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;

/**
 * Managed Bean for EPLStatement entity.  
 * This class also takes care about adding listeners to statements
 * 
 * @author Štefan Repček
 */
@Named("eplStatementManagedBean")
@Scope("session")
public class EPLStatementManagedBean implements Serializable {

    private final static Log log = LogFactory.getLog(EPLStatementManagedBean.class);
   

    @Autowired
    private EPLStatementService eplStatementService;
    private EPLStatementDTO eplStatement = new EPLStatementDTO();  
    private List<EPLStatementDTO> eplStatements;
    private List<String> listenerLabels = new ArrayList<String>();
    private List<String> listenerExpressions = new ArrayList<String>();
    private List<String> targetForExpressions = new ArrayList<String>();
    private List<String> targetForLabels = new ArrayList<String>();
    private List<String> targetRowForExpressions = new ArrayList<String>();
    private String eplStatementNameToRemove;
    private boolean isPattern;

    public List<String> getTargetRowForExpressions() {
        return targetRowForExpressions;
    }
    
    public boolean getIsPattern() {
        return isPattern;
    }
    
    public List<EPLStatementDTO> getEplStatements() {
        return eplStatements;
    }

    public void setEplStatements(List<EPLStatementDTO> eplStatements) {
        this.eplStatements = eplStatements;
    }
    
    public String getEplStatementNameToRemove() {
        return eplStatementNameToRemove;
    }

    public void setEplStatementNameToRemove(String eplStatementNameToRemove) {
        this.eplStatementNameToRemove = eplStatementNameToRemove;
    }

    public void setIsPattern(boolean isPattern) {
        this.isPattern = isPattern;
    }

    public void setTargetRowForExpressions(List<String> targetRowForExpressions) {
        this.targetRowForExpressions = targetRowForExpressions;
    }

    public List<String> getTargetForLabels() {
        return targetForLabels;
    }

    public void setTargetForLabels(List<String> oldLabels) {
        this.targetForLabels = oldLabels;
    }

    public List<String> getTargetForExpressions() {
        return targetForExpressions;
    }

    public void setTargetForExpressions(List<String> oldExpressions) {
        this.targetForExpressions = oldExpressions;
    }

    public List<String> getListenerLabels() {
        return listenerLabels;
    }

    public void setListenerLabels(List<String> listenerLabels) {
        this.listenerLabels = listenerLabels;
    }

    public List<String> getListenerExpressions() {
        return listenerExpressions;
    }

    public void setListenerExpressions(List<String> listenerExpressions) {
        this.listenerExpressions = listenerExpressions;
    }
    
    public void setEplStatementService(EPLStatementService eplStatementService) {
        this.eplStatementService = eplStatementService;
    }
  

    public EPLStatementDTO getEplStatement() {
        return eplStatement;
    }

    public void setEplStatement(EPLStatementDTO eplStatement) {
        this.eplStatement = eplStatement;
    }

    public EPLStatementManagedBean() {
    }

    public void addListener() {
        listenerLabels.add(null);
        listenerExpressions.add(null);
        targetForExpressions.add(null);
        targetForLabels.add(null);
        targetRowForExpressions.add("0");


    }



    @PostConstruct
    public void init() {
        eplStatements = eplStatementService.findAllEPLStatements();
    }

    public void add() {
        if (BackendHelpers.validateEPLStatement(eplStatement.getEplStatement(), eplStatement.getName())) {
            try {
                eplStatement.setId(null);
                eplStatementService.createEPLStatement(eplStatement);


                log.info("EPL Statement saved");
                init();
            } catch (PreexistingEntityException ex) {
                log.error(ex);

                BackendHelpers.sendFacesMessage("eplStatement", ex.getMessage());
            }

        }


    }
    /*
     *  This method  save listener to listener String
     *  Listener String consist of Labels and Expressions which are divided by special characters(&§@~)
     */
    public void saveListener() throws IOException {
        String listener = "";
        if(validateListener())
        {
        for (int i = 0; i < listenerExpressions.size(); i++) {

            if (targetForLabels.get(i).equals(String.valueOf(true))) {
                listener += "§";
            }
            listener += listenerLabels.get(i) + "&";
            if (targetForExpressions.get(i).equals(String.valueOf(true))) {
                listener += "§";
            }
            listener += targetRowForExpressions.get(i) + "~";
            listener += listenerExpressions.get(i);
            if (i != listenerExpressions.size() - 1) {
                listener += "@";
            }


        }
        eplStatement.setListener(listener);
        FacesContext.getCurrentInstance().getExternalContext().redirect("eplstatements.xhtml");
        }
        else
        {
            BackendHelpers.sendFacesMessage("listener_form","'&', '@', '~', '§' are reservered characters");           
        }
    }

    public void removeProperty() {
        listenerLabels.remove(listenerLabels.size() - 1);
        listenerExpressions.remove(listenerExpressions.size() - 1);
        targetForExpressions.remove(targetForExpressions.size() - 1);
        targetForLabels.remove(targetForLabels.size() - 1);
        targetRowForExpressions.remove(targetRowForExpressions.size() - 1);
        



    }

    public void save() {
        if (BackendHelpers.validateEPLStatement(eplStatement.getEplStatement(), eplStatement.getName())) {
            if (eplStatement.getId() != null) {
                try {
                    eplStatementService.updateEPLStatement(eplStatement);
                    init();
                    log.info("EPL Statement udpated");

                } catch (PreexistingEntityException ex) {
                    log.error(ex);
                    BackendHelpers.sendFacesMessage("eplStatement", ex.getMessage());
                }
            } else {
                BackendHelpers.sendFacesMessage("eplStatement", "statement " + eplStatement.getName() + " is not stored in database");
            }
        }
    }

    public void delete() throws NonexistentEntityException {
        Map<String, String> params = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();
        EPLStatementDTO eplStatementToDelete = eplStatementService.getEPLStatementbyID(Long.parseLong(params.get("id")));
        eplStatementService.deleteEPLStatement(eplStatementToDelete);
        init();

        log.info("EPL Statement deleted");
    }

    public void select() {
        Map<String, String> params = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();
        eplStatement = eplStatementService.getEPLStatementbyID(Long.parseLong(params.get("id")));
        listenerLabels = new ArrayList<String>();
        listenerExpressions = new ArrayList<String>();
        
        targetForExpressions = new ArrayList<String>();
        targetForLabels = new ArrayList<String>();
        targetRowForExpressions = new ArrayList<String>();
        if (eplStatement.getListener() != null && !"".equals(eplStatement.getListener())) {
            String[] rows = eplStatement.getListener().split("@");
            int size = rows.length;
            for (int i = 0; i < size; i++) {
                String[] pair = rows[i].split("&", -1);
                if (pair[0].charAt(0) == '§') {
                    targetForLabels.add(String.valueOf(true));
                    pair[0] = pair[0].replace("§", "");
                } else {
                    targetForLabels.add(String.valueOf(false));
                }
                listenerLabels.add(pair[0]);
                
                if (pair[1].charAt(0) == '§') {
                    targetForExpressions.add(String.valueOf(true));
                    pair[1] = pair[1].replace("§", "");
                } else {
                    targetForExpressions.add(String.valueOf(false));
                }

                String[] pair2 = pair[1].split("~", -1);
                targetRowForExpressions.add(pair2[0]);
                listenerExpressions.add(pair2[1]);
            }
        }
    }

    public void searchByName(String name) {
        eplStatementService.findEPLStatementByName(name);

    }

  
    
    private boolean validateListener()
    {
        for(String s:listenerLabels)
        {
            if(s.contains("&") || s.contains("§") || s.contains("~") || s.contains("@"))
            {
                return false;
            }
        }
        for(String s: listenerExpressions)
        {
            if(s.contains("&") || s.contains("§") || s.contains("~") || s.contains("@"))
            {
                return false;
            }
        }
        
        return true;
    }

  
}
