/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.cepportal.database.dao.impl;

import cz.muni.fi.cepportal.database.dao.EPLStatementDAO;
import cz.muni.fi.cepportal.database.entities.EPLStatement;
import cz.muni.fi.cepportal.database.exceptions.NonexistentEntityException;
import cz.muni.fi.cepportal.database.exceptions.PreexistingEntityException;
import java.io.Serializable;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import org.springframework.stereotype.Repository;



/**
 * This class realises CRUD operations with EPLStatement entity.
 * 
 * @author Štefan Repček
 */

@Repository
public class EPLStatementDAOImpl implements EPLStatementDAO, Serializable{
    
    @PersistenceContext(unitName="CEPPortalPU")
    private EntityManager em;

    public void setEm(EntityManager em) {
        this.em = em;
    }

    @Override
    public void createEPLStatement(EPLStatement eplStatement) throws PreexistingEntityException {
        if(eplStatement == null)
        {
            throw new IllegalArgumentException("statement is null");
        }
        if(this.findEPLStatementByName(eplStatement.getName())!=null)
        {
            throw new PreexistingEntityException("statement with name " + eplStatement.getName() + " already exist" );
        }
        em.persist(eplStatement);
      
     
    }

    @Override
    public void updateEPLStatement(EPLStatement eplStatement) throws PreexistingEntityException   {
        if(eplStatement == null)
        {
            throw new IllegalArgumentException("statement is null");
        }
       
        if(this.findEPLStatementByName(eplStatement.getName())!= null && this.findEPLStatementByName(eplStatement.getName()).getId() != eplStatement.getId())
        {
            throw new PreexistingEntityException("statement with name " + eplStatement.getName() + " already exist" );
        }
  
        em.merge(eplStatement);
    }

    @Override
    public void deleteEPLStatement(EPLStatement eplStatement) throws NonexistentEntityException {
        if(eplStatement == null)
        {
            throw new IllegalArgumentException("statement is null");
        }
         if(this.getEPLStatementbyID(eplStatement.getId()) == null)
        {
            throw new NonexistentEntityException("statement with name " + eplStatement.getName() + " doesnt exist" );
        }
         em.remove(em.merge(eplStatement));
    }

    @Override
    public EPLStatement getEPLStatementbyID(Long id) {
        if(id == null)
        {
            throw new IllegalArgumentException("id is null");
        }
        return em.find(EPLStatement.class,id);
    }

    @Override
    public EPLStatement findEPLStatementByName(String name) {
        Query q = em.createQuery("select object(o) from EPLStatement as o where o.name='"+ name + "'");
      
        
        List<EPLStatement> list = q.getResultList();
        if(list.isEmpty())
        {
            return null;
        }
        else
        {
            return list.get(0);
        }
        
    }

    @Override
    public List<EPLStatement> findAllEPLStatements() {
        Query q = em.createQuery("select object(o) from EPLStatement as o");
        return q.getResultList();
    }
    
}
