/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.cepportal.database.dao.impl;


import cz.muni.fi.cepportal.database.dao.EventStreamDAO;
import cz.muni.fi.cepportal.database.entities.EventStream;
import cz.muni.fi.cepportal.database.exceptions.NonexistentEntityException;
import cz.muni.fi.cepportal.database.exceptions.PreexistingEntityException;
import java.io.Serializable;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import org.springframework.stereotype.Repository;

/**
 * This class realises CRUD operations with EventStream entity.
 * 
 * @author Štefan Repček
 */
@Repository
public class EventStreamDAOImpl implements EventStreamDAO, Serializable{

    
    @PersistenceContext
    private EntityManager em;

    public void setEm(EntityManager em) {
        this.em = em;
    }
    
    
    @Override
    public void createEventStream(EventStream eventStream) throws PreexistingEntityException {
         if(eventStream == null)
        {
            throw new IllegalArgumentException("event stream is null");
        }
        if(this.findEventStreamByName(eventStream.getName())!=null)
        {
            throw new PreexistingEntityException("event stream with name " + eventStream.getName() + " already exist" );
        }
        em.persist(eventStream);
    }

    @Override
    public void updateEventStream(EventStream eventStream) throws PreexistingEntityException {
         if(eventStream == null)
        {
            throw new IllegalArgumentException("eventStream is null");
        }
       
        if(this.findEventStreamByName(eventStream.getName())!= null && this.findEventStreamByName(eventStream.getName()).getId() != eventStream.getId())
        {
            throw new PreexistingEntityException("eventStream with name " + eventStream.getName() + " already exist" );
        }
  
        em.merge(eventStream);
    }

    @Override
    public void deleteEventStream(EventStream eventStream) throws NonexistentEntityException {
        if(eventStream == null)
        {
            throw new IllegalArgumentException("eventStream is null");
        }
         if(this.getEventStreamByID(eventStream.getId()) == null)
        {
            throw new NonexistentEntityException("eventStream with name " + eventStream.getName() + " doesnt exist" );
        }
         em.remove(em.merge(eventStream));
    }

    @Override
    public EventStream getEventStreamByID(Long id) {
         if(id == null)
        {
            throw new IllegalArgumentException("id is null");
        }
        return em.find(EventStream.class,id);
    }

    @Override
    public EventStream findEventStreamByName(String name) {
        Query q = em.createQuery("select object(o) from EventStream as o where o.name='"+ name + "'");
      
        
        List<EventStream> list = q.getResultList();
        if(list.isEmpty())
        {
            return null;
        }
        else
        {
            return list.get(0);
        }
    }

    @Override
    public List<EventStream> findAllEventStreams() {
        Query q = em.createQuery("select object(o) from EventStream as o");
        return q.getResultList();
    }
    
}
