/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.cepportal.database.dao.impl;

import cz.muni.fi.cepportal.database.dao.EventTypeDAO;
import cz.muni.fi.cepportal.database.entities.EventType;
import cz.muni.fi.cepportal.database.exceptions.NonexistentEntityException;
import cz.muni.fi.cepportal.database.exceptions.PreexistingEntityException;
import java.io.Serializable;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import org.springframework.stereotype.Repository;

/**
 * This class realises CRUD operations with EventType entity.
 * 
 * @author Štefan Repček
 */
@Repository
public class EventTypeDAOImpl implements EventTypeDAO, Serializable {

    
    @PersistenceContext
    private EntityManager em;

    public void setEm(EntityManager em) {
        this.em = em;
    }

    
    @Override
    public void createEventType(EventType eventType) throws PreexistingEntityException {
        if(eventType == null)
        {
            throw new IllegalArgumentException("event is null");
        }
        if(this.findEventTypeByName(eventType.getEventName())!=null)
        {
            throw new PreexistingEntityException("event with name " + eventType.getEventName() + " already exist" );
        }
        em.persist(eventType);
    }

    @Override
    public void updateEventType(EventType eventType) throws PreexistingEntityException {
        if(eventType == null)
        {
            throw new IllegalArgumentException("event is null");
        }
       
        if(this.findEventTypeByName(eventType.getEventName())!= null && this.findEventTypeByName(eventType.getEventName()).getId() != eventType.getId())
        {
            throw new PreexistingEntityException("event with name " + eventType.getEventName() + " already exist" );
        }
  
        em.merge(eventType);
    }

    @Override
    public void deleteEventType(EventType eventType) throws NonexistentEntityException {
        if(eventType == null)
        {
            throw new IllegalArgumentException("event is null");
        }
         if(this.getEventTypeByID(eventType.getId()) == null)
        {
            throw new NonexistentEntityException("event with name " + eventType.getEventName() + " doesnt exist" );
        }
         em.remove(em.merge(eventType));
    }

    @Override
    public EventType getEventTypeByID(Long id) {
         if(id == null)
        {
            throw new IllegalArgumentException("id is null");
        }
        return em.find(EventType.class,id);
    }

    @Override
    public EventType findEventTypeByName(String name) {
        Query q = em.createQuery("select object(o) from EventType as o where o.eventName='"+ name + "'");
      
        
        List<EventType> list = q.getResultList();
        if(list.isEmpty())
        {
            return null;
        }
        else
        {
            return list.get(0);
        }
    }

    @Override
    public List<EventType> findAllEventTypes() {
        Query q = em.createQuery("select object(o) from EventType as o");
        return q.getResultList();
    }
    
}
