/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.cepportal.database.services.impl;

import cz.muni.fi.cepportal.database.dto.CEPOutputDTO;
import cz.muni.fi.cepportal.database.dto.convert.CEPOutputConverter;
import cz.muni.fi.cepportal.database.dao.CEPOutputDAO;
import cz.muni.fi.cepportal.database.entities.CEPOutput;
import cz.muni.fi.cepportal.database.exceptions.NonexistentEntityException;
import cz.muni.fi.cepportal.database.services.CEPOutputService;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Service class which handles DAO exceptions and manages transaction
 * 
 * @author Štefan Repček
 */
@Service
public class CEPOutputServiceImpl implements CEPOutputService, Serializable{

    private static Log log = LogFactory.getLog(CEPOutputServiceImpl.class);
    
    @Autowired
    private CEPOutputDAO cepOutputDAO;

    public void setEplStatementDAO(CEPOutputDAO eplStatementDAO) {
        this.cepOutputDAO = eplStatementDAO;
    }
    
    @Override
    @Transactional
    public void createCEPOutput(CEPOutputDTO cepOutput) {
      try
      {
        cepOutputDAO.createCEPOutput(CEPOutputConverter.dtoToEntity(cepOutput));
      }
      catch(DataAccessException ex)
      {
          log.error(ex.getMessage());
          throw ex;
      }
    }

    @Override
    @Transactional
    public void updateCEPOutput(CEPOutputDTO cepOutput) {
        try
      {
        cepOutputDAO.updateCEPOutput(CEPOutputConverter.dtoToEntity(cepOutput));
      }
      catch(DataAccessException ex)
      {
          log.error(ex.getMessage());
          throw ex;
      }
    }

    @Override
    @Transactional
    public void deleteCEPOutput(CEPOutputDTO cepOutput) throws NonexistentEntityException {
      try
      {
        cepOutputDAO.deleteCEPOutput(CEPOutputConverter.dtoToEntity(cepOutput));
      }
      catch(DataAccessException ex)
      {
          log.error(ex.getMessage());
          throw ex;
      }
    }

    @Override
    @Transactional(readOnly=true)
    public CEPOutputDTO getCEPOutputByID(Long id) {
        try
      {
        return CEPOutputConverter.entityToDTO(cepOutputDAO.getCEPOutputByID(id));
      }
      catch(DataAccessException ex)
      {
          log.error(ex);
          throw ex;
      }
    }

    @Override
    @Transactional(readOnly=true)
    public CEPOutputDTO findCEPOutputByName(String name) {
       try
      {
        return CEPOutputConverter.entityToDTO(cepOutputDAO.findCEPOutputByName(name));
      }
      catch(DataAccessException ex)
      {
          log.error(ex);
          throw ex;
      }
    }

    @Override
    @Transactional(readOnly=true)
    public List<CEPOutputDTO> findAllCEPOutputs() {
        List <CEPOutputDTO> dtos = new ArrayList<CEPOutputDTO>();
        
        for(CEPOutput d : cepOutputDAO.findAllCEPOutputs())
        {
            CEPOutputDTO dto = CEPOutputConverter.entityToDTO(d);
            dtos.add(dto);
            
        }
        
        return dtos;
    }
    
}
