/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.cepportal.database.services.impl;

import cz.muni.fi.cepportal.database.dto.EventStreamDTO;
import cz.muni.fi.cepportal.database.dto.convert.EventStreamConverter;
import cz.muni.fi.cepportal.database.dao.EventStreamDAO;
import cz.muni.fi.cepportal.database.entities.EventStream;
import cz.muni.fi.cepportal.database.exceptions.NonexistentEntityException;
import cz.muni.fi.cepportal.database.exceptions.PreexistingEntityException;
import cz.muni.fi.cepportal.database.services.EventStreamService;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Service class which handles DAO exceptions and manages transaction
 * 
 * @author Štefan Repček
 */
@Service
public class EventStreamServiceImpl implements EventStreamService, Serializable{

     private static Log log = LogFactory.getLog(EventStreamServiceImpl.class);
    
    @Autowired
    private EventStreamDAO eventStreamDAO;
    
    
    
    @Override
    @Transactional
    public void createEventStream(EventStreamDTO eventStream) throws PreexistingEntityException {
      try
      {
        eventStreamDAO.createEventStream(EventStreamConverter.dtoToEntity(eventStream));
      }
      catch(DataAccessException ex)
      {
          log.error(ex.getMessage());
          throw ex;
      }
    }

    @Override
    @Transactional
    public void updateEventStream(EventStreamDTO eventStream) throws PreexistingEntityException {
        try
      {
        eventStreamDAO.updateEventStream(EventStreamConverter.dtoToEntity(eventStream));
      }
      catch(DataAccessException ex)
      {
          log.error(ex.getMessage());
          throw ex;
      }
    }

    @Override
    @Transactional
    public void deleteEvent(EventStreamDTO eventStream) throws NonexistentEntityException {
        try
      {
        eventStreamDAO.deleteEventStream(EventStreamConverter.dtoToEntity(eventStream));
      }
      catch(DataAccessException ex)
      {
          log.error(ex.getMessage());
          throw ex;
      }
    }

    @Override
    @Transactional(readOnly=true)
    public EventStreamDTO getEventStreamByID(Long id) {
         try
      {
        return EventStreamConverter.entityToDTO(eventStreamDAO.getEventStreamByID(id));
      }
      catch(DataAccessException ex)
      {
          log.error(ex);
          throw ex;
      }
    }

    @Override
    @Transactional(readOnly=true)
    public EventStreamDTO findEventStreamByName(String name) {
       try
      {
        return EventStreamConverter.entityToDTO(eventStreamDAO.findEventStreamByName(name));
      }
      catch(DataAccessException ex)
      {
          log.error(ex);
          throw ex;
      }
    }

    @Override
    public List<EventStreamDTO> findAllEventStreams() {
          List <EventStreamDTO> dtos = new ArrayList<EventStreamDTO>();
        
        for(EventStream s : eventStreamDAO.findAllEventStreams())
        {
            EventStreamDTO dto = EventStreamConverter.entityToDTO(s);
            dtos.add(dto);
            
        }
        
        return dtos;
    }
    
}
