/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.cepportal.database.services.impl;

import cz.muni.fi.cepportal.database.dto.EventTypeDTO;
import cz.muni.fi.cepportal.database.dto.convert.EventTypeConverter;
import cz.muni.fi.cepportal.database.dao.EventTypeDAO;
import cz.muni.fi.cepportal.database.entities.EventType;
import cz.muni.fi.cepportal.database.exceptions.NonexistentEntityException;
import cz.muni.fi.cepportal.database.exceptions.PreexistingEntityException;
import cz.muni.fi.cepportal.database.services.EventTypeService;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Service class which handles DAO exceptions and manages transaction
 * 
 * @author Štefan Repček
 */
@Service
public class EventTypeServiceImpl implements EventTypeService, Serializable {

    private static Log log = LogFactory.getLog(EventTypeServiceImpl.class);
    
    @Autowired
    private EventTypeDAO eventDAO;
    
    
    
    @Override
    @Transactional
    public void createEventType(EventTypeDTO eventType) throws PreexistingEntityException {
       try
      {
        eventDAO.createEventType(EventTypeConverter.dtoToEntity(eventType));
      }
      catch(DataAccessException ex)
      {
          log.error(ex.getMessage());
          throw ex;
      }
    }

    @Override
    @Transactional
    public void updateEventType(EventTypeDTO eventType) throws PreexistingEntityException {
       try
      {
        eventDAO.updateEventType(EventTypeConverter.dtoToEntity(eventType));
      }
      catch(DataAccessException ex)
      {
          log.error(ex.getMessage());
          throw ex;
      }
    }

    @Override
    @Transactional
    public void deleteEventType(EventTypeDTO eventType) throws NonexistentEntityException {
         try
      {
        eventDAO.deleteEventType(EventTypeConverter.dtoToEntity(eventType));
      }
      catch(DataAccessException ex)
      {
          log.error(ex.getMessage());
          throw ex;
      }
    }

    @Override
    @Transactional(readOnly=true)
    public EventTypeDTO getEventTypeByID(Long id) {
       try
      {
        return EventTypeConverter.entityToDTO(eventDAO.getEventTypeByID(id));
      }
      catch(DataAccessException ex)
      {
          log.error(ex);
          throw ex;
      }
    }

    @Override
    @Transactional(readOnly=true)
    public EventTypeDTO findEventTypeByName(String name) {
      try
      {
        return EventTypeConverter.entityToDTO(eventDAO.findEventTypeByName(name));
      }
      catch(DataAccessException ex)
      {
          log.error(ex);
          throw ex;
      }
    }

    @Override
    public List<EventTypeDTO> findAllEventTypes() {
        List <EventTypeDTO> dtos = new ArrayList<EventTypeDTO>();
        
        for(EventType s : eventDAO.findAllEventTypes())
        {
            EventTypeDTO dto = EventTypeConverter.entityToDTO(s);
            dtos.add(dto);
            
        }
        
        return dtos;
    }
    
    
}
